'This is a demo of the PMP TV driver
'Copyright (C) 2012 Rayslogic.com, LLC
'The PMP_TV driver is made to work much like the original Parallax TV driver but for this 2.4" LCD screen.
'This demo also shows how to make a sizeable and moveable graphics area
'This demo is in landscape mode so touchscreen x and y axis are flipped

'Adapted Ray's code 27/03/2013 by skylight
'When first started up program allows screen colour scheme look to be changed
'The colour of the Frame and the Font can be changed by
'Home icon = advance next colour scheme
'Email icon = go back to previous colour scheme
'holding either button scrolls through schemes
'Note icon = Accept scheme

'Once Note icon is pressed the paint program is active as below:

'Allow line thickness to be altered from 1 to 16 pixels
'Also changed layout of button functions as follows:
'Home icon = Red
'Email icon = Green
'Book icon = Blue
'Phone icon = Line thickness(1-16) tap to change line width by a pixel or hold down to scroll.
'Note icon = Screen Clear


CON

  _clkmode = xtal1 + pll16x
  _xinfreq =5_000_000               
  'Double buffering is not needed because PSM's GRAM serves as the second buffer
  'So, 12kB ($3000 bytes in hex) is saved. 
  '_stack = ($3000 + $3000 + 100) >> 2   'accomodate display memory and stack
  _stack = (x_tiles*y_tiles*16*4 + 128) >> 2   'accomodate display memory and stack
     'In full screen mode, need to reserve 19,200 bytes (4,800 longs) 
  x_tiles = 18 'PMP can do up to 20   
  y_tiles = 11 'PMP can do up to 15    

  paramcount = 14
  'Since we're not double-buffering, pointing both these display buffer pointers to the same place
  'Shifting the bitmap_base pointer to the end of memory, where display_base also points 
  bitmap_base = ($8000)-x_tiles*y_tiles*16*4-128
  display_base = bitmap_base

  lines = 5
  thickness = 2

  cols = 20
  rows = 15

  screensize = cols * rows

  GraphicsXOffset=1
  GraphicsYOffset=3
  

VAR

  long  mousex, mousey

  long  tv_status     '0/1/2 = off/visible/invisible           read-only
  long  tv_enable     '0/? = off/on                            write-only
  long  tv_pins       '%ppmmm = pins                           write-only
  long  tv_mode       '%ccinp = chroma,interlace,ntsc/pal,swap write-only
  long  tv_screen     'pointer to screen (words)               write-only
  long  tv_colors     'pointer to colors (longs)               write-only               
  long  tv_hc         'horizontal cells                        write-only
  long  tv_vc         'vertical cells                          write-only
  long  tv_hx         'horizontal cell expansion               write-only
  long  tv_vx         'vertical cell expansion                 write-only
  long  tv_ho         'horizontal offset                       write-only
  long  tv_vo         'vertical offset                         write-only
  long  tv_broadcast  'broadcast frequency (Hz)                write-only
  long  tv_auralcog   'aural fm cog                            write-only

  word  screen[rows*cols]
  long  colors[64]

  byte  x[lines]
  byte  y[lines]
  byte  xs[lines]
  byte  ys[lines]

  long  row,col,flag,color

  long touchx, touchy
  long CurrentColor, bLineMode, bStillDown, LastX, LastY  

OBJ

  tv    : "QMP_TV"            
  gr    : "graphics"
  i2c: "QMP_I2CDriver" 
 

PUB start | i, dx,dy,GraphicsColor
  'start tv
  longmove(@tv_status, @tvparams, paramcount)
  tv_screen := @screen
  tv_colors := @colors
  tv.start(@tv_status)
  tv.StartAutoUpdate 'This will keep updating the LCD screen automatically for us 
  
  
  setcolors(@palette) 
  out(0) 'clear screen
  col:=0
  row:=0
  
  Str(string("Paint Line Width= "))
  
        

    'init colors
  GraphicsColor:=16 'Using this color for graphics colors
  colors[GraphicsColor]:=$E8389800  'red, blue, magenta, black
  
  'init tile screen
  repeat dx from 0 to x_tiles - 1
    repeat dy from 0 to y_tiles - 1
      screen[(dy+GraphicsYOffset) * cols + dx+GraphicsXOffset] := display_base >> 6 + dy + dx * y_tiles +GraphicsColor<<10


  'start and setup graphics
  gr.start
  gr.setup(x_tiles, y_tiles, 0, 0, bitmap_base)  'bottom-left corner is origin

  gr.color(2)     'Initial stylus color
  gr.width(1)     'Stylus width
  
  Paint

Pub Paint |xc,yc,n,width,colour
  'Calculate coordinate of bottom left corner of graphics area
  xc:=GraphicsXOffset*16
  yc:=(rows-GraphicsYOffset-y_Tiles)*16
  width:=1
   col := 17
   row := 0
  out(dec(width))
   col:=18
   out(" ")
   col:=19
   out(" ")

  repeat 
    WaitStylusDown
    if touchy<0  'this is one of the virtual buttons
      n:=touchx*5/240
      case n
        4:
          setcolors(colour)
          WaitStylusUp
         waitcnt(cnt+clkfreq/4)
         colour++
        3:
          setcolors(colour)
          WaitStylusUp
         waitcnt(cnt+clkfreq/4)
         colour--
         
        0:  
         quit 


        
  repeat
    WaitStylusDown
    if touchy<0  'this is one of the virtual buttons
      n:=touchx*5/240
      case n
        4:
          gr.color(2)      '2=Red
          col := 0
          row := 0
          Str(string("Red  "))
          
        3:
          gr.color(3)      '3=Green
           col := 0
          row := 0
          Str(string("Green"))

          
        2:
          gr.color(1)      '1=Blue
           col := 0
          row := 0
          Str(string("Blue "))
          
        1:
         WaitStylusUp
         waitcnt(cnt+clkfreq/10)
        
           
          gr.width(width++)    'Increase stylus thickness
          
          if width>16
           width:=1
          
          col := 17
          row := 0
           out(dec(width))
           
           if width<10
            col :=18
            out(" ")
            col :=19
            out(" ")
           if width>9
            col :=19
            out(" ")

        0:
              
          gr.clear         'Clear Screen
         
    gr.plot(touchy-xc,touchx-yc)
   
  
    

 

PUB bTouchDown|tx,ty,z1,z2,tz  
  tx:=i2c.GetTouchX
  z1:=i2c.GetTouchZ1
  z2:=i2c.GetTouchZ2
  tz:=tx*(z2-z1)/z1
  if (tz<90000) and (tz>10000)           'You might have to tweak these values for some displays...
    return true
  else
    return false

PUB WaitStylusUp
  'Wait for stylus to be up
  repeat
    waitcnt(cnt+clkfreq/100)  
    if !bTouchDown
    return

    
PUB WaitStylusDown|tx,ty,z1,z2,tz
  'Wait for stylus to be down then update touchx, touchy and return
  repeat
    if bTouchDown
      tx:=i2c.GetTouchX
      ty:=i2c.GetTouchY
      z1:=i2c.GetTouchZ1
      z2:=i2c.GetTouchZ2
      'take another x,y reading and average
      tx+=i2c.GetTouchX
      ty+=i2c.GetTouchY
      tx>>=1
      ty>>=1
      tz:=tx*(z2-z1)/z1
      if (tz<90000) and (tz>10000)
        touchx:=(tx-touchx0)/touchdx
        touchy:=(ty-touchy0)/touchdy
        return
      else
        bStillDown:=false
    else
      bStillDown:=false    


DAT 'Default Touchscreen Calibrations
'based on these reading:
'  Top of landscape screen: x=60600, Bottom of landscape screen: x=4560   
'  Left of landscape screen (screen only, not buttons): y=6200,  Right of landscape screen:  y=60600
'  Left of button area:  (y=1500)  
touchx0 long 4560
touchdx long (60600-4560)/240
touchy0 long 6200
touchdy long (60600-6200)/320

PUB out(c) | i, k

'' Output a character
''
''     $00 = clear screen
''     $01 = home
''     $08 = backspace
''     $09 = tab (8 spaces per)
''     $0A = set X position (X follows)
''     $0B = set Y position (Y follows)
''     $0C = set color (color follows)
''     $0D = return
''  others = printable characters

  case flag
    $00: case c
           $00: wordfill(@screen, $220, screensize)
                col := row := 0
           $01: col := row := 0
           $08: if col
                  col--
           $09: repeat
                  print(" ")
                while col & 7
           $0A..$0C: flag := c
                     return
           $0D: newline
           other: print(c)
    $0A: col := c // cols
    $0B: row := c // rows
    $0C: color := c & 7
  flag := 0
  
PRI print(c)|k,i
      k := color << 1 + c & 1
      i := k << 10 + $200 + c & $FE
      screen[row * cols + col] := i
      screen[(row + 1) * cols + col] := i | 1
      if ++col == cols

PRI newline : i

  col := 0
  if (row += 2) == rows
    row -= 2
    'scroll lines
    repeat i from 0 to rows-3
'      wordmove(@screen[i*cols], @screen[(i+2)*cols], cols)
    'clear new line
'    wordfill(@screen[(rows-2)*cols], $200, cols<<1)

PUB str(stringptr)

'' Print a zero-terminated string

  repeat strsize(stringptr)
    out(byte[stringptr++])


PUB dec(value) | i

'' Print a decimal number

  if value < 0
    -value
    out("-")

  i := 1_000_000_000

  repeat 10
    if value => i
      out(value / i + "0")
      value //= i
      result~~
    elseif result or i == 1
      out("0")
    i /= 10


PUB hex(value, digits)

'' Print a hexadecimal number

  value <<= (8 - digits) << 2
  repeat digits
    out(lookupz((value <-= 4) & $F : "0".."9", "A".."F"))


PUB bin(value, digits)

'' Print a binary number

  value <<= 32 - digits
  repeat digits
    out((value <-= 1) & 1 + "0")




PUB setcolors(colorptr) | i, fore, back

'' Override default color palette
'' colorptr must point to a list of up to 8 colors
'' arranged as follows:
''
''               fore   back
''               ------------
'' palette  byte color, color     'color 0
''          byte color, color     'color 1
''          byte color, color     'color 2
''          ...

  repeat i from 0 to 7
    fore := byte[colorptr][i << 1]
    back := byte[colorptr][i << 1 + 1]
    colors[i << 1]     := fore << 24 + back << 16 + fore << 8 + back
    colors[i << 1 + 1] := fore << 24 + fore << 16 + back << 8 + back


DAT

tvparams                long    0               'status
                        long    1               'enable
                        long    %001_0101       'pins
                        long    %0000           'mode
                        long    0               'screen
                        long    0               'colors
                        long    cols         'hc
                        long    rows         'vc
                        long    10              'hx
                        long    1               'vx
                        long    0               'ho
                        long    0               'vo
                        long    0               'broadcast
                        long    0               'auralcog

DAT
                        '       fore   back
                        '       color  color
palette                 byte    $07,   $0A    '0    white / dark blue
                        byte    $07,   $BB    '1    white / red
                        byte    $9E,   $9B    '2   yellow / brown
                        byte    $04,   $07    '3     grey / white
                        byte    $3D,   $3B    '4     cyan / dark cyan
                        byte    $6B,   $6E    '5    green / gray-green
                        byte    $BB,   $CE    '6      red / pink
                        byte    $3C,   $0A    '7     cyan / blue